#version 330 core

in vec2 texCoord;
in vec3 lightDir;
in vec3 viewDir;
in vec3 halfVec;
in vec3 ambientCol;
in vec3 lightCol;
in vec4 shadowCoord;

layout(location = 0)out vec4 fragColor;


uniform vec3 kD; 		//diffuse Reflection
uniform vec3 kS; 		//specular Reflection
uniform int shininess;

uniform sampler2D shadow;
uniform sampler2D diffuse1;
uniform sampler2D normal1;
uniform sampler2D specular1;


vec3 phongModel(vec3 norm, vec3 diffR, float visibility)
{
	vec3 ambient = ambientCol * lightCol;	
	float sDotN = clamp( dot(lightDir, norm), 0.0,1.0);
	vec3 diffuse = lightCol * kD * sDotN;
	
	vec3 spec = vec3(0.0);
	if(sDotN > 0.0 && shininess>0.0)
	{
		spec = lightCol * kS *
			pow( clamp( dot(halfVec,norm)*0.8, 0.0,1.0), 20);
	}
	return ambient*diffR + (diffuse*diffR + spec)*visibility;
}

float offsetLookup(vec2 uv, vec2 offset)
{
	return texture(shadow, uv+offset*1.0/2048.0).z;
}

float pcf(vec2 uv, float d, float bias)
{
	float vis = 1.0f;
	for(float x = -1.5; x<=1.5; x++)
	{
		for(float y = -1.5; y<=1.5; y++)
		{
			if(offsetLookup(uv,vec2(x,y)) < d-bias)
			{
				vis -= 1.0/16.0;
			}
		}
	}
	return vis;
}

void main()
{
    vec4 texColor = texture(diffuse1, texCoord);
	vec3 normal = texture(normal1, texCoord).xyz;
	float visibility = 1.0f;
	float sh = pcf(shadowCoord.xy, shadowCoord.z, 0.005f);

    fragColor = vec4(phongModel(normal,texColor.rgb,sh),texColor.a);
}