#version 330 core
layout(location = 0)in vec3 vertexPosition;
layout(location = 1)in vec3 vertexNormal;
layout(location = 2)in vec3 vertexTexCoord;
layout(location = 3)in vec4 vertexTangent;

layout(std140) uniform CameraInfo
{
	mat4 projectionMat;
	mat4 viewMat;
};

layout(std140) uniform LightingInfo
{
	vec3 lightPosition;
	vec3 ambientColor;
	vec3 lightColor;
};

uniform mat4 modelMat;
uniform mat3 normalMat;
uniform mat4 depthBiasPVMat;

out vec2 texCoord;
out vec3 lightDir;
out vec3 ambientCol;
out vec3 lightCol;
out vec3 viewDir;
out vec3 halfVec;
out vec4 shadowCoord;
out vec3 vertexPos;

void main() {
	
	vec3 norm = normalize(normalMat * vertexNormal);
	vec3 tang = normalize(normalMat * vertexTangent.xyz);
	vec3 binormal = normalize(cross(norm,tang)) * vertexTangent.w;
	
	mat3 tnb = mat3(
		tang.x, binormal.x, norm.x,
		tang.y, binormal.y, norm.y,
		tang.z, binormal.z, norm.z);
	
	vec3 pos = vec4(viewMat * modelMat* vec4(vertexPosition,1)).xyz;
	
	vertexPos = vertexPosition;
	ambientCol = ambientColor;
	lightCol = lightColor;
	texCoord = vertexTexCoord.xy;
	lightDir = tnb* normalize(lightPosition);
	viewDir = tnb * normalize(-pos);
	halfVec = normalize(viewDir+lightDir);
	gl_Position = projectionMat * viewMat * modelMat* vec4(vertexPosition,1);
	shadowCoord = depthBiasPVMat * modelMat * vec4(vertexPosition,1);
}