#version 330 core

in vec2 texCoord;
in vec3 lightDir;
in vec3 viewDir;
in vec3 halfVec;
in vec4 shadowCoord;
in vec3	vertexPos;
layout(location = 0)out vec4 fragColor;

layout(std140) uniform LightingInfo
{
	vec3 lightPosition;
	vec3 ambientColor;
	vec3 lightColor;
};

uniform vec3 kD; 		//diffuse Reflection
uniform vec3 kS; 		//specular Reflection
uniform int shininess;
uniform vec3 color;

uniform sampler2D shadow;
uniform sampler2D diffuse1;
uniform sampler2D normal1;

uniform float height;
uniform float fill;

vec4 phongModel(vec3 norm, float spec, vec3 diffR, float visibility)
{
	vec3 ambient = ambientColor * lightColor;	
	float sDotN = clamp( dot(lightDir, norm), 0.0,1.0);
	vec3 diffuse = lightColor * sDotN;
	
	if(vertexPos.y > height*fill)
		diffR = vec3(0.5);
	
	vec3 s = vec3(0.0);
	if(sDotN > 0.0)
	{
		s = lightColor * 0.8 *
			pow( clamp( dot(halfVec,norm)*spec*1.3, 0.0,1.0), 10);
	}
	return vec4((ambient*diffR) + ((diffuse*diffR + s))*visibility,sDotN);
}

float offsetLookup(vec2 uv, vec2 offset)
{
	return texture(shadow, uv+offset*1.0/2048.0).x;
}

float pcf(vec2 uv, float d, float bias)
{
	float vis = 1.0f;
	for(float x = -1.5; x<=1.5; x++)
	{
		for(float y = -1.5; y<=1.5; y++)
		{
			if(offsetLookup(uv,vec2(x,y)) < d-bias)
			{
				vis -= 1.0/16.0;
			}
		}
	}
	return vis;
}

void main()
{
    vec4 texColor = texture(diffuse1, texCoord);
	vec4 normal = texture(normal1, texCoord);
	float visibility = 1.0f;
	float sh = pcf(shadowCoord.xy, shadowCoord.z, 0.005f);
	vec4 col = phongModel(normal.xyz,normal.w,texColor.rgb,sh);
	
	float f = 0;
	if(vertexPos.y/height > 0.95)
		f = 0.5f;
	col = mix(col,vec4(color,1), f);
	
    fragColor = vec4(col.xyz,1);
}