#version 330 core
layout(location = 0)in vec3 vertexPosition;
layout(location = 1)in vec3 vertexNormal;
layout(location = 2)in vec2 vertexTexCoord0;
layout(location = 3)in vec2 vertexTexCoord1;
layout(location = 4)in vec3 vertexIndizes;

uniform mat4 modelMat;
uniform mat3 normalMat;
uniform mat4 depthBiasPVMat;
uniform int isSelection;
layout(std140) uniform CameraInfo
{
	mat4 projectionMat;
	mat4 viewMat;
};

layout(std140) uniform LightingInfo
{
	vec3 lightPosition;
	vec3 ambientColor;
	vec3 lightColor;
};

out vec2 texCoord;
out vec2 texCoord1;
out vec3 normal;
out vec3 indizes;
out vec3 lightDir;
out vec3 viewDir;
out vec3 halfVec;
out vec4 shadowCoord;

void main() {
	texCoord = vertexTexCoord0;
	texCoord1 = vertexTexCoord1;
	indizes = vertexIndizes;
	vec3 vp = vertexPosition;

	vec3 norm = normalize(normalMat * vertexNormal).xyz;
	vec3 tang = normalize(normalMat * vec3(vertexNormal.x,0,-vertexNormal.z));
	vec3 binormal = normalize(cross(norm,tang));
	vec3 pos = vec4(viewMat * modelMat* vec4(vp,1)).xyz;

	mat3 tnb = mat3(
	tang.x, binormal.x, norm.x,
	tang.y, binormal.y, norm.y,
	tang.z, binormal.z, norm.z);
	lightDir = tnb* normalize(lightPosition);
	viewDir = tnb * normalize(-pos);
	halfVec = normalize(viewDir+lightDir);
	normal = norm;
	shadowCoord = depthBiasPVMat * modelMat * vec4(vertexPosition,1);

	gl_Position = projectionMat * viewMat * modelMat* vec4(vp,1);
}